#ifndef _MARCHE_HPP_
#define _MARCHE_HPP_

#include <random> // Pour simuler des variables aléatoires. Nécessite le standard C++ 2011 (-std=c++11).
#include <cmath>
#include <vector>

class marche
{
private:
  int N;
  int x;
  int y;
  int t;
  std::vector<std::vector<int> > histo;
public:
  // Un constructeur. histo(N_) fait en sorte que histo soit un vecteur de taille N_,
  // dont les entrées sont la valeur par défaut de la classe std::vector<int>,
  // c'est-à-dire un vecteur vide. On a donc N_ vecteurs vide histo[0], ... histo[N-1].
  marche(int N_) : N(N_), x(0), y(0), t(0), histo(N_)
  {}

  // On n'a pas besoin de définir une constructeur par copie, un operateur = et un destructeur, car il n'y a pas
  // de manipulation de la mémoire dans cette classe.
  

  
  // On fournit un générateur en argument aux fonctions qui nécessiterons de l'aléa.
  void un_pas(std::mt19937 &gen);
  void r_a_z(void);
  void sortie(std::mt19937 &gen);
  void ajoute_histo(void);

  //Un accesseur de la variable histo. On doit déclarer l'objet courant comme constant, ainsi que la valeur de retour.
  const std::vector<std::vector<int> >& hist(void) const;

};
#include <iostream>
void marche::un_pas(std::mt19937 &gen)
{
  if (std::abs(x)!=N or std::abs(y)!=N)
	{
	  t ++;
	  std::uniform_int_distribution<int> X(0,3);
	  int alea = X(gen); // On tire un nombre entre 0 et 3 (en utilisant le générateur fourni en argument).
	  // Les quatre résultats, 0, 1, 2 et 3 ont probabilité 1/4.

	  // Une manière d'associer à chaque résultat 0,1,2 ou 3 un vecteur (1,0), (0,1), (-1,0), (0,-1).
	  if (alea%2==0)
		x += 2*(alea/2)-1;
	  else
		y += 2*(alea/2) - 1;
	}
}

// On remet à zéro les trois champs t, x et y.
void marche::r_a_z(void)
{
  t=0;
  x=0;
  y=0;
}

// Une méthode qui simule la trajectoire jusqu'à sa sortie de {-N+1,...,N-1}^2.
void marche::sortie(std::mt19937 &gen)
{
  while (std::abs(x)!=N and std::abs(y)!=N)
	un_pas(gen);
}

// Une méthode qui comptabilise les temps de sortie dans la variable histo.
void marche::ajoute_histo(void)
{
  if (std::abs(x)==N)
	histo[std::abs(y)].push_back(t);
  else
	histo[std::abs(x)].push_back(t);  
}

// Un accesseur pour histo
const std::vector<std::vector<int> >& marche::hist(void) const
{
  return histo;
}



#endif
